import javax.swing.Timer.*;
import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

public class GamePlayer extends JFrame implements ActionListener {
    GameSimulator theSimulator;
    static Maze aMaze;
    MazeGraphics mg;
    MazePanel mazePanel;
    TextPanel textPanel;
    int fps = 5;
    int gameSize = 40;
    int delay;
    javax.swing.Timer timer;
    boolean gameOver = false;
    static Random rg;
    
    public GamePlayer(long seed) {
        Container contentPane = getContentPane();
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
	BevelBorder aBorder = new BevelBorder(1);
        contentPane.setLayout(gridbag);

	// get rid of this?
	enableEvents(AWTEvent.KEY_EVENT_MASK);

	// deal with the timer
	delay = (fps > 0)?(1000/fps):1000;
	timer = new javax.swing.Timer(delay,this);
	//timer.setInitialDelay(1000);
	timer.setCoalesce(true);

	//System.out.println(seed);
	setRandom(seed);

	c.fill = GridBagConstraints.BOTH;

	// prepare the simulator
	aMaze = new Maze(gameSize, gameSize);
	theSimulator = new GameSimulator(aMaze);
	theSimulator.setInitialState();

	// prepare the graphics components
	mg = new MazeGraphics(aMaze, theSimulator.simState);
	mazePanel = new MazePanel(mg);
	textPanel = new TextPanel(theSimulator.simState);
	//JPanel controlPanel = new GameControlPanel(this);

	mazePanel.setBorder(aBorder);
        c.gridx = 0;        c.gridy = 0;
	c.gridheight = 1;	c.gridwidth = 1;
	c.weightx = c.weighty = 1.0;
        c.ipady = 40;      //make this component tall
        contentPane.add(mazePanel, c);

        c.ipadx = 40;
        c.gridx = 1;  c.gridy = 0;
	c.gridheight = 1; c.gridwidth = 1;
	c.weightx = c.weighty = 0.1;
        contentPane.add(textPanel, c);

        addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    Quit();
		}
	    });
    }

    public void setFPS(int fps) {
	this.fps = fps;
	delay = (fps > 0)?(1000/fps):1000;
	timer.setDelay(delay);
    }

    public void StartAnimation() {
	if (!gameOver && !timer.isRunning()) {
	    timer.start();
	} 
    }

    public void StopAnimation () {
	if (timer.isRunning()) {
	    timer.stop();
	}
    }

    public void ResetAnimation() {
	gameOver = false;
	if (timer.isRunning()) {
	    timer.stop();
	}
	theSimulator.setInitialState();
	mg.resetMaze(aMaze);
	mazePanel.UpdateWith(theSimulator.simState);
	textPanel.UpdateWith(theSimulator.simState);
    }

    public void IncrementAnimation() {
	if (theSimulator.CanDoAnotherStep()) {
	    theSimulator.Step();
	    mazePanel.UpdateWith(theSimulator.simState);
	    textPanel.UpdateWith(theSimulator.simState);
	}
	else {
	    StopAnimation();
	    Toolkit.getDefaultToolkit().beep();
	    gameOver = true;
	    theSimulator.PostSimulationStep(this);
	}
    }

    static Maze getMaze(){
	return aMaze;
    }

    public void Quit() {
	StopAnimation();
	gameOver = true;
	//dispose();
	System.exit(0);
    }
    
    /** Is the handler for the animation step, which prompts the simulator
     *	to update and then the graphics + text panels 
     */
    public void actionPerformed(ActionEvent e) {
	IncrementAnimation();
	/*
	if (theSimulator.CanDoAnotherStep()) {
	    theSimulator.Step();
	    mazePanel.UpdateWith(theSimulator.simState);
	    textPanel.UpdateWith(theSimulator.simState);
	}
	else {
	    StopAnimation();
	    Toolkit.getDefaultToolkit().beep();
	    gameOver = true;
	    theSimulator.PostSimulationStep(this);
	}
	*/
    }

    public void processKeyEvent(KeyEvent e) {
	if (e.getID() == KeyEvent.KEY_PRESSED) {
	    if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
		fps++;
		timer.setDelay(1000/fps);
	    }
	    else if (e.getKeyCode() == KeyEvent.VK_LEFT) {
		if (fps > 1) {
		    fps--;
		    timer.setDelay(1000/fps);
		}
	    }
	    else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
		if (!timer.isRunning())
		    timer.start();
		else
		    timer.stop();
	    }
	    else if (e.getKeyCode() == KeyEvent.VK_R) {
		ResetAnimation();
	    }
	    else if (e.getKeyCode() == KeyEvent.VK_I) {
		IncrementAnimation();
	    }
	}
    }

    private void setRandom(long seed) {
	if (seed > 0)
	    rg = new Random(seed);
	else
	    rg = new Random();
    }

    public static int nextInt(int n) { return rg.nextInt(n); }
    public static float nextFloat() { return rg.nextFloat(); }

    public static Random getRandom() { return rg; }

    public static void main(String args[]) throws IOException {
	GamePlayer window;
	if (args.length > 0)
	    window = new GamePlayer((new Long(args[0])).longValue());
	else
	    window = new GamePlayer(0);
        //window.inAnApplet = false;
	
        window.setTitle("The GAME");
        window.pack();
        window.setVisible(true);
	//window.setRandom(0);
	//window.StartAnimation();
    }
}
