import java.util.*;
import java.io.*;


/**
  This is the Maze class.  It stores the maze and has
  routines for initializing the maze.  Importantly, it also
  stores the MazeObjectTypes, which contain info on how to display
  and simulate the objects in the maze. 
*/
public class Maze {
    /** width in cells */
    int width;
    /** height in cells */
    int height;
    /** A list of walls */
    int[][] walls;
    int numWalls;
    /** A list of MazeObjectTypes */
    Vector objectTypes;
    /** bare constructor */
    public Maze() {
	objectTypes = new Vector();
	numWalls = 0;
    }
    
    /** Creates a Maze of width w and height h */
    public Maze(int w, int h) {
	objectTypes = new Vector();
	numWalls = 0;
	width = w;
	height = h;
	walls = new int[width][height];
    }

    public int getHeight() {
	return height;
    }

    public int getWidth() {
	return width;
    }

    public int getNumWalls() {
	return numWalls;
    }

    /** set the size of the maze */
    public void setSize(int w, int h) {
	width = w;
	height = h;
	walls = new int[width][height];
    }

    public boolean squareInDir(int x, int y, int dir, Position p) {
	int newx, newy;
	newx = x;
	newy = y;
	switch (dir) {
	case 0: newy--; break;
	case 1: newx++; break;
	case 2: newy++; break;
	case 3: newx--; break;
	default:
	}
	if (CoordsAreInMaze(newx, newy) && !wallIsBetween(x,y,newx,newy)) {
	    p.x = newx;	p.y = newy;
	    return true;
	}
	else
	    return false;
    }


    public boolean CoordsAreInMaze(int x,int y) {
	return (x>=0 && x<width && y>=0 && y<height);
    }

    public boolean wallIsBetween(int x1, int y1, int x2, int y2) {
	int x,y, lookfor;
	if (x1 != x2) {
	    lookfor = 1;
	    y = y1;
	    x = Math.min(x1,x2);
	}
	else {
	    lookfor = 2;
	    x = x1;
	    y = Math.min(y1,y2);
	}
	int code = walls[x][y];
	if (code == lookfor || code == 3)
	    return true;
	else
	    return false;
    }

    /** This should fill in the walls randomly, using the 
     * <br> density parameter to choose how many walls.  
     * <br> Density ranges between 0.0 and 1.0 and should 
     * <br> be some indication of the percentage of walls 
     * <br> that are there.  We would like a fully connected maze,
     * <br> however, so this function must be somewhat careful;
     */
    public void RandomInitWalls(float density) {
	//Random rg = new Random();
	for (int i=0; i<this.width; i++) {
	    for (int j=0; j<this.height; j++) {
		// no walls in player's square
		if (i == 0 && j == 0)
		    continue;
		if (i<this.width-1 && GamePlayer.nextFloat() < density) {
		    walls[i][j] = 1;
		    numWalls++;
		}
		else
		    walls[i][j] = 0;
		if (j<this.height-1 && GamePlayer.nextFloat() < density) {
		    numWalls++;
		    walls[i][j] += 2;
		}
	    }
	}
    }

}


