/*
 * 1.2 version.
 */
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import java.awt.image.*;
import javax.swing.*;


/** 
 * This is the object that actually draws the maze and the objects within it.
 * This object paints the canvas for the maze panel,
 * as well as storing the info for what to paint.  
 * It gets updated when the objects move, for example.
 */
public class MazeGraphics {
    /** The size of a square in the graphics2D "virtual space" */
    static final int squareSize=50;
    /** The offset around the maze */
    static final int offset = 100;

    /** the height and width of the maze*/
    public int height, width;
    /** Shape used to draw the maze walls */
    Shape aShape;
    /** Shape used to draw the boundary of the drawing area */
    Shape boundaryShape;
    /** Shape used to draw the boundary of the maze */
    Shape mazeBoundaryShape;
    GeneralPath aPath;  // this is the maze path. 
    GeneralPath aBoundary; // this is the boundary of the maze region
    GeneralPath mazeBoundary; // this is the boundary of the maze region

    //final static BasicStroke stroke = new BasicStroke(2.0f);
    public final static BasicStroke wideStroke = new BasicStroke(8.0f);

    //final static float dash1[] = {10.0f};
    /*final static BasicStroke dashed = new BasicStroke(1.0f, 
                                                      BasicStroke.CAP_BUTT, 
                                                      BasicStroke.JOIN_MITER, 
                                                      10.0f, dash1, 0.0f);
    */
    public Maze maze;
    public Vector currentObjects;
    public Vector lastObjects;
    public int morigin_x, morigin_y;
    public int mwidth, mheight;
    /** Constructor: takes the maze and the current list of
        MazeObjects as input */
    public MazeGraphics(Maze daMaze, SimulationState simState) {
	maze = daMaze;
	lastObjects = null;
	currentObjects = simState.objects;
	aBoundary = new GeneralPath();
	width = squareSize*maze.width + offset*2;
	height = squareSize*maze.height + offset*2;
	morigin_x = offset; 	morigin_y = offset; 
	mheight = height - offset; mwidth = width-offset;
	aBoundary.append(new Line2D.Float(0,0,0,width),false);
	aBoundary.append(new Line2D.Float(0,width,height,width),false);
	aBoundary.append(new Line2D.Float(height,width,height,0),false);
	aBoundary.append(new Line2D.Float(height,0,0,0),false);
	boundaryShape = aBoundary;
	mazeBoundary = new GeneralPath();
	mazeBoundary.append(new Line2D.Float(morigin_x, morigin_y, morigin_x, mheight),false);
	mazeBoundary.append(new Line2D.Float(morigin_x, morigin_y, mwidth, morigin_y),false);
	mazeBoundary.append(new Line2D.Float(mwidth, mheight, morigin_x, mheight),false);
	mazeBoundary.append(new Line2D.Float(mwidth, mheight, mwidth, morigin_y),false);
	mazeBoundaryShape = mazeBoundary;
	aPath = new GeneralPath();
	setMazeWalls();
    }

    public void resetMaze(Maze daMaze) {
	maze = daMaze;
	setMazeWalls();
    }

    private void setMazeWalls() {
	aPath.reset();
	int h = maze.getHeight();
	int w = maze.getWidth();
	for (int x=0; x<w; x++) {
	    for (int y=0; y<h; y++) {
		if (x<w-1 && maze.wallIsBetween(x,y,x+1,y)) {
		    aPath.append(new 
			Line2D.Float((x+1)*squareSize + morigin_x,
				     y*squareSize + morigin_y,
				     (x+1)*squareSize + morigin_x,
				     (y+1)*squareSize + morigin_y), false);
		}
		if (y<h-1 && maze.wallIsBetween(x,y,x,y+1)) {
		    aPath.append(new 
			Line2D.Float(x*squareSize + morigin_x,
				     (y+1)*squareSize + morigin_y,
				     (x+1)*squareSize + morigin_x,
				     (y+1)*squareSize + morigin_y), false);
		}
	    }
	}
	aShape = aPath;
    }

    /** Updates the mazePanel graphics for the new vector of objects */
    public void UpdateWith(SimulationState simState, MazePanel mPanel) {
	lastObjects = currentObjects;
	// now repaint last..., 
	// and make the new objects!
	currentObjects = simState.objects;
	// and finally repaint the new...
	mPanel.repaint();
    }

    /** the actual paint routine, called by the maze panel */
    public void paint(Graphics2D g2, MazePanel mPanel) {
	g2.setColor(Color.black);
	g2.draw(boundaryShape); // the boundary
	g2.draw(mazeBoundaryShape); // the boundary of the maze
	g2.draw(aShape);  // the maze itself

	// Now need to add in the drawing of the objects...
	for (Iterator i=currentObjects.iterator();i.hasNext();) {
	    MazeObject obj = (MazeObject)(i.next());
	    if (obj.visible) {
		int drawx = offset + squareSize*obj.x();
		int drawy = offset + squareSize*obj.y();
		obj.draw(g2, drawx, drawy, squareSize, mPanel);
	    }
	}
    }
}
