import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import java.lang.*;
import java.awt.geom.*;

/**
 * This the class of objects that are instantiations of
 * MazeObjectTypes.  They are what are dealt with in the simulator,
 * and these can be added and deleted.  They explicitly reference
 * their MazeObjectType, and are what are drawn as well. 
 */
public abstract class MazeObject {
    /** The location of the object in the maze */
    private int x, y;
    
    /** whether the object is slated for deletion */
    public boolean delete;

    public PerceptObject po;

    /** whether the object is visible or not */
    public boolean visible;

    /** the image */
    public BufferedImage image;

    /** A constructor that initializes position to the origin */
    public MazeObject() {
	this(0, 0);
    }

    /** A constructor that creates an object with a particular position */
    public MazeObject(int x, int y) {
	this.x = x; this.y = y; delete = false;
	po = new PerceptObject();
	po.x = po.lastx = x;
	po.y = po.lasty = y;
	po.special = -1;
	visible = true;
    }

    /** moves the object to a particular location */
    public void Move(int x, int y) {
	po.lastx = this.x;
	po.lasty = this.y;
	this.x = po.x = x;
	this.y = po.y = y;
    }

    public void setx(int x) {
	po.lastx = this.x;
	this.x = po.x = x;
    }

    public void sety(int y) {
	po.lasty = this.y;
	this.y = po.y = y;
    }

    public int x() {
	return x;
    }
    public int y() {
	return y;
    }

    /** returns the position of the object */
    public Position getPosition() {
	return new Position(this.x, this.y);
    }

    // returns the next move of the object
    public abstract int nextMove();

    // draws the object on g2, in the box with upper left x,y
    protected String imageFileName;
    public void draw(Graphics2D g2, int x, int y, int squareSize, JPanel p) {
	int offset = squareSize / 10;
        if (image == null) {
	    Image i = new javax.swing.ImageIcon(imageFileName).getImage();
	    BufferedImage tmpImage = 
		(BufferedImage)p.createImage(i.getWidth(p),
					     i.getHeight(p));
	    Graphics2D g = tmpImage.createGraphics();
	    g.drawImage(i, 0, 0, p);

	    // scale the image to the square size
	    AffineTransformOp op = new AffineTransformOp
		(AffineTransform.
		 getScaleInstance((squareSize-2*offset)/
				  (double)tmpImage.getWidth(), 
				  (squareSize-2*offset)/
				  (double)tmpImage.getHeight()),
		 AffineTransformOp.TYPE_BILINEAR);
	    image = op.filter(tmpImage, null);
	}
        g2.drawImage(image, null, x+offset, y+offset);
    }
}
