import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.geom.*;

public class Player extends MazeObject implements Comparator {

    // The object that chooses commands:
    PlayerActor pa;
    PlayerPercept pp;

    // the commands:
    int command;
    boolean dropDoughnut;
    boolean eatDoughnut;

    int doughnutCount;
    int energy;
    int inspiration;
    int thinking;
    int thinktime;
    int id;
    String name;
    int cluesSolved;
    Clue clue;
    Position cluePos;
    Cop holdingCop;
    int finishtime;
    int noMove;

    private int held;

    public Player() {
	this(0,0,0, "Minivan.gif");
    }

    public Player(int id, String imgFileName) {
	this(0,0, id, imgFileName);
    }

    public Player(int x, int y, int id, String imgFileName) {
	super(x,y);
	this.command = -1;
	this.doughnutCount = 2;
	this.energy = 100;
	this.inspiration = 80;
	this.thinking = 0;
	this.thinktime = 0;
	this.held = 0;
	this.dropDoughnut = false;
	this.eatDoughnut = false;
	this.cluesSolved = -1;
	this.clue = null;
	this.cluePos = null;
	this.holdingCop = null;
	this.id = id;
	this.finishtime = 0;
	this.noMove = 0;
	this.imageFileName = imgFileName;
	this.name = "Player " + Integer.toString(id);
	po.type = PerceptObject.PLAYER;
    }

    public Position getCluePosition() {
	return cluePos;
    }

    public void solveClue(Clue nextClue) {
	clue = nextClue;
	cluePos = nextClue.getPosition();
	cluesSolved++;
    }

    public void foundClue(Clue c) {
	clue = null;
	cluePos = null;
	c.found++;
    }

    public int getCluesSolved() {
	return cluesSolved;
    }

    public LocalMap getCluePlan() {
	if (clue != null)
	    return clue.cluePlan;
	else
	    return null;
    }

    public void setHeld(int n) {
	held = n;
	if (n == 0)
	    po.special = -1;
	else 
	    po.special = n;
    }

    public int held() {
	return held;
    }

    public int nextMove() { 
	if (held > 0)
	    return -1;
	else
	    return command;
    }

    public void act() {
	pa.act();
    }

    public void pickUpDoughnut() {
	doughnutCount++;
    }

    public int compare(Object o1, Object o2) {
	if (((Player)o1).cluesSolved < ((Player)o2).cluesSolved)
	    return 1;
	else if (((Player)o1).cluesSolved > ((Player)o2).cluesSolved)
	    return -1;
	else
	    return 0;
    }

    public boolean equals(Object obj) {
	if (((Player)obj).cluesSolved == cluesSolved)
	    return true;
	else
	    return false;
    }
}
