import java.io.*;
import java.util.*;


/** PlayerActor <BR> 

    This is that class that you must subclass.  You should name your
    subclass TeamNameActor.java, where you obviously replace TeamName
    with your name (with no punctuation or spaces, please).  You must
    use only the functions in this class and those in PlayerInterface
    to access information about the game -- you must not call any
    other functions in the game code, except for objects that are
    returned by the functions below (these are explained in more
    detail below as well).  You also may not override any of these
    functions except as specified below.  Also, you should not create
    new PlayerInterface objects -- these are created for you in the
    PlayerActor constructor.

    Your player actor will be called by the simulator by calling the
    function act().  You must override this method with your own code.
    This function is a void function; you must call the functions
    setDir(int dir), drop(), and/or eat() (which are in
    PlayerInterface) to affect the environment.  By default, the
    direction command is the same as it was on the last turn (as in
    the video game), and the agent will only drop or eat doughnuts
    when specifically told to do so.

    Your player has access to perceptual information from the
    environment, through PlayerInterface.

    You may add new utility methods if you desire to call from act();

    The directions use the convention that 0=NORTH, 1=EAST, 2=SOUTH,
    3=WEST, and 4=STOP.  There are constants for these values below.

    By using the functions below and in PlayerInterface, you have access
    to enough features of the environment to write a decent agent.
    One is provided for you in ExampleActor.java.  You can use
    this agent as is, (although this may cost you something both in
    your placement at the start of the game and in the quality of your
    gaming experience), or you can use it as a base to write your
    agent.

 */

public abstract class PlayerActor { 
    /**** Code that isn't to be changed or overridden ***********/
    public final int NORTH = 0;
    public final int EAST  = 1; 
    public final int SOUTH = 2;
    public final int WEST  = 3;
    public final int STOP  = 4;
    PlayerInterface pi;
    Random rg;
    public PlayerActor(PlayerInterface p) {
	pi = p;
	rg = GamePlayer.getRandom();
    }
    /********************End of unchangeable code****************/

    /*********** This is the function you must write ************/

    public abstract void act();

    /************************************************************/
}
